from rest_framework import serializers
from pandas import DataFrame


class PandasBaseSerializer(serializers.Serializer):
    """
    Transforms dataset into a dataframe and appies an index
    """
    read_only = True

    def get_index(self, dataframe):
        return None

    def get_dataframe(self, data):
        dataframe = DataFrame(data)
        index = self.get_index(dataframe)
        if index:
            dataframe = dataframe.set_index(index)
        else:
            # Name auto-index column to ensure valid CSV output
            dataframe.index.name = 'row'
        return dataframe

    def transform_dataframe(self, dataframe):
        view = self.context.get('view', None)
        if view and hasattr(view, 'transform_dataframe'):
            return self.context['view'].transform_dataframe(dataframe)
        return dataframe

    @property
    def data(self):
        data = super(PandasBaseSerializer, self).data
        if data:
            dataframe = self.get_dataframe(data)
            return self.transform_dataframe(dataframe)
        else:
            return DataFrame([])


class PandasSimpleSerializer(PandasBaseSerializer):
    """
    Simple serializer for non-model (simple) views
    """
    def get_default_fields(self):
        if not self.object:
            return {}
        return {
            name: serializers.Field()
            for name in self.object[0].keys()
        }


class PandasSerializer(PandasBaseSerializer, serializers.ModelSerializer):
    """
    Serializer for model views.
    """
    def get_index(self, dataframe):
        return ['id']



from rest_framework.renderers import BaseRenderer
from rest_framework import status
from tempfile import mkstemp
from StringIO import StringIO
import os


class PandasBaseRenderer(BaseRenderer):
    """
    Renders DataFrames using their built in Pandas implementation.
    Only works with serializers that return DataFrames as their data object.
    Uses a StringIO to capture the output of dataframe.to_[format]()
    """
    def render(self, data, accepted_media_type=None, renderer_context=None):
        if 'response' in renderer_context:
            status_code = renderer_context['response'].status_code
            if not status.is_success(status_code):
                return "Error: %s" % data.get('detail', status_code)

        name = getattr(self, 'function', "to_%s" % self.format)
        function = getattr(data, name, None)
        if not function:
            raise Exception("Data frame is missing %s property!" % name)
        self.init_output()
        args = self.get_pandas_args(data)
        kwargs = self.get_pandas_kwargs(data)
        function(*args, **kwargs)
        return self.get_output()

    def init_output(self):
        self.output = StringIO()

    def get_output(self):
        return self.output.getvalue()

    def get_pandas_args(self, data):
        return [self.output]

    def get_pandas_kwargs(self, data):
        return {}


class PandasFileRenderer(PandasBaseRenderer):
    """
    Renderer for output formats that absolutely must use a file (i.e. Excel)
    """
    def init_output(self):
        file, filename = mkstemp(suffix='.' + self.format)
        self.filename = filename

    def get_pandas_args(self, data):
        return [self.filename]

    def get_output(self):
        result = open(self.filename).read()
        os.unlink(self.filename)
        return result


class PandasCSVRenderer(PandasBaseRenderer):
    """
    Renders data frame as CSV
    """
    media_type = "text/csv"
    format = "csv"


class PandasTextRenderer(PandasBaseRenderer):
    """
    Renders data frame as CSV, but uses text/plain as media type
    """
    media_type = "text/plain"
    format = "txt"
    function = "to_csv"


class PandasJSONRenderer(PandasBaseRenderer):
    """
    Renders data frame as JSON
    """
    media_type = "application/json"
    format = "json"

    def get_pandas_kwargs(self, data):
        return {
            'orient': 'index',
            'date_format': 'iso',
        }


class PandasExcelRenderer(PandasFileRenderer):
    """
    Renders data frame as Excel (.xlsx)
    """
    media_type = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
    format = "xlsx"
    function = "to_excel"


class PandasOldExcelRenderer(PandasFileRenderer):
    """
    Renders data frame as Excel (.xls)
    """
    media_type = "application/vnd.ms-excel"
    format = "xls"
    function = "to_excel"


class PandasImageRenderer(PandasBaseRenderer):
    """
    Renders dataframe using built-in plot() function
    """
    function = "plot"

    def init_output(self):
        import matplotlib.pyplot as plt
        self.fig = plt.figure()
        self.ax = self.fig.add_subplot(111)

    def get_pandas_args(self, data):
        return []

    def get_pandas_kwargs(self, data):
        return {'ax': self.ax}

    def get_output(self):
        data = StringIO()
        self.fig.savefig(data, format=self.format)
        return data.getvalue()


class PandasPNGRenderer(PandasImageRenderer):
    media_type = "image/png"
    format = "png"


class PandasSVGRenderer(PandasImageRenderer):
    media_type = "image/svg"
    format = "svg"



from rest_framework.views import APIView
from rest_framework.generics import ListAPIView
from rest_framework.mixins import ListModelMixin
from rest_framework.viewsets import GenericViewSet
from rest_framework.response import Response

from django.conf import settings
from rest_framework.settings import perform_import

from .serializers import PandasSimpleSerializer, PandasSerializer

PANDAS_RENDERERS = getattr(settings, "PANDAS_RENDERERS", None)
if PANDAS_RENDERERS is None:
    PANDAS_RENDERERS = (
        "rest_pandas.renderers.PandasCSVRenderer",
        "rest_pandas.renderers.PandasTextRenderer",
        "rest_pandas.renderers.PandasJSONRenderer",
        "rest_pandas.renderers.PandasExcelRenderer",
        "rest_pandas.renderers.PandasOldExcelRenderer",
        "rest_pandas.renderers.PandasPNGRenderer",
        "rest_pandas.renderers.PandasSVGRenderer",
    )

PANDAS_RENDERERS = perform_import(PANDAS_RENDERERS, "PANDAS_RENDERERS")


class PandasSimpleView(APIView):
    """
    Simple (non-model) Pandas API view; override get_data
    with a function that returns a list of dicts.
    """
    serializer_class = PandasSimpleSerializer
    renderer_classes = PANDAS_RENDERERS

    def get_data(self, request, *args, **kwargs):
        return []

    def get(self, request, *args, **kwargs):
        data = self.get_data(request, *args, **kwargs)
        serializer = self.serializer_class(data, many=True)
        return Response(serializer.data)


class PandasView(ListAPIView):
    """
    Pandas-capable model list view
    """
    model_serializer_class = PandasSerializer
    renderer_classes = PANDAS_RENDERERS
    paginate_by = None


class PandasViewSet(ListModelMixin, GenericViewSet):
    """
    Pandas-capable model ViewSet (list only)
    """
    model_serializer_class = PandasSerializer
    renderer_classes = PANDAS_RENDERERS
    paginate_by = None



from os.path import join, dirname, exists
import sys
import subprocess
from setuptools import setup, find_packages

LONG_DESCRIPTION = """
Serves up Pandas dataframes via the Django REST Framework for client-side (i.e. d3.js) visualizations
"""


def parse_markdown_readme():
    """
    Convert README.md to RST via pandoc, and load into memory
    (fallback to LONG_DESCRIPTION on failure)
    """
    # Check for existing file (Pandoc's RST tables don't work with rst2html...)
    path = join(dirname(__file__), 'README.rst')
    if not exists(path):
        # Attempt to run pandoc on markdown file
        try:
            subprocess.call(
                ['pandoc', '-t', 'rst', '-o', 'README.rst', 'README.md']
            )
        except OSError:
            return LONG_DESCRIPTION

    # Attempt to load output
    try:
        readme = open(path)
    except IOError:
        return LONG_DESCRIPTION
    return readme.read()

setup(
    name='rest-pandas',
    version='0.1.0',
    author='S. Andrew Sheppard',
    author_email='andrew@wq.io',
    url='https://github.com/wq/django-rest-pandas',
    license='MIT',
    packages=['rest_pandas'],
    description=LONG_DESCRIPTION.strip(),
    long_description=parse_markdown_readme(),
    install_requires=[
        'djangorestframework>=2.3.12',
        'pandas>=0.13.0',
    ],
    classifiers=[
        'Framework :: Django',
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Visualization',
    ],
    test_suite='tests.runtests.main',
    tests_require=['wq.io'],
)
